package frc.robot.utilities;

import java.util.function.Consumer;
import java.util.function.DoubleSupplier;

import com.revrobotics.spark.SparkBase;

import edu.wpi.first.wpilibj.Encoder;
import edu.wpi.first.wpilibj.motorcontrol.MotorController;

public class EncoderMotorHelper {
    private MotorController controller;

    private DoubleSupplier positionSource;
    
    private Consumer<Double> positionSetter;

    public EncoderMotorHelper(MotorController controller, Encoder encoder) {
        this.controller = controller;

        positionSource = encoder::getDistance;
        positionSetter = (d) -> encoder.reset();
    }

    public EncoderMotorHelper(SparkBase spark, boolean useAbsolutePosition) {
        controller = spark;

        if (useAbsolutePosition) {
            positionSource = spark.getAbsoluteEncoder()::getPosition;
            positionSetter = (d) -> {}; // TODO Do nothing position setter doesn't make a ton of sense...
        } else {
            positionSource = spark.getEncoder()::getPosition;
            positionSetter = spark.getEncoder()::setPosition;
        }
    }

    public EncoderMotorHelper(MotorController controller, DoubleSupplier positionSource, Consumer<Double> positionSetter) {
        this.controller = controller;
        this.positionSource = positionSource;
        this.positionSetter = positionSetter;
    }

    public MotorController getMotorController() {
        return controller;
    }

    public DoubleSupplier getPositionSource() {
        return positionSource;
    }

    public Consumer<Double> getPositionSetter() {
        return positionSetter;
    }
}
